#include <functional>
#include <iostream>
#include <vector>

class Object
{
public:
    int    id;
    double x;

    static Object None;
};
Object Object::None {-1,-1.0};

template<class F>
Object& find_if(std::vector<Object>& a, const F& c)
{
    for(auto& i : a)
    {
        if(c(i))
            return i;
    }
    return Object::None;
}

class ObjectComparison
{
public:
    ObjectComparison(int p_id){ id = p_id; }
    bool operator()(const Object& o) const { return o.id == id; }
    int id;
};

class X
{
public:
    void setEventHandler(std::function<void(int)>){}
    std::function<void(int)> fctOnEvent;
};

Object& find_if_lambda(std::vector<Object>& a, const std::function<bool(Object&)> c)
{
    for(auto& i : a)
    {
        if(c(i))
            return i;
    }
    return Object::None;
}

int main(int, char**)
{
    std::vector<Object> v { {1, 1.0}, {2, 2.0} };

    std::cout << "find '2': " << find_if(v, ObjectComparison(2)).id << std::endl;
    std::cout << "find '3': " << find_if(v, ObjectComparison(3)).id << std::endl;

    std::cout << "lambda find '2': " << find_if_lambda(v, ObjectComparison(2)).id << std::endl;
    std::cout << "lambda find '3': " << find_if_lambda(v, ObjectComparison(3)).id << std::endl;

    /// Lambda
    std::cout << "lambda find '3': " << find_if_lambda(v, [](Object& a){ return a.id == 3; }).id << std::endl;

    /// Lambda with closure
    int id = 2;
    std::cout << "lambda find '3': " << find_if_lambda(v, [id](Object& a){ return a.id == id; }).id << std::endl;

    X x;
    x.setEventHandler( [&v](int id){ v.push_back({id, 0.0}); } );
}
